@@ -84,6 +84,21 @@ x & col_name\\\\
   )
 )
 
+assert('kable(format = "latex", linesep = ...) works', {
+  kable2(data.frame(x = 1:4), 'latex', linesep  = c("", "", "\\midrule")) %==% "
+\\begin{tabular}{r}
+\\hline
+x\\\\
+\\hline
+1\\\\
+2\\\\
+3\\\\
+\\midrule
+4\\\\
+\\hline
+\\end{tabular}"
+})
+
 assert(
   'kable() escapes HTML special characters by default',
   identical(
@@ -142,6 +157,14 @@ assert(
   identical(kable2(x2, 'markdown'), c('|a  |', '|:--|'))
 )
 
+assert(
+  'kable(, "pandoc") works for a 0 row data.frame',
+  identical(
+    kable2(data.frame(x = character(0), y = integer(0)), 'pandoc'),
+    c('|x  |  y|', '|:--|--:|')
+  )
+)
+
 assert('kable(, "pandoc", caption = "Table Caption") works for a 1-column matrix', {
   x4 = matrix(1:2, ncol = 1, dimnames = list(NULL, 'a'))
   kable2(x4, 'pandoc', caption = 'Table Caption') %==%
